"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const one_shot_1 = require("piper-js/one-shot");
//import { FeatureList } from 'piper-js/core';
const time_1 = require("piper-js/time");
const qm_1 = require("@extractors/qm");
// this spawns a web worker, which we only want to do once
// so we instantiate
const qmWorker = qm_1.default();
function bufferToAudioData(buffer) {
    const nChannels = buffer.numberOfChannels;
    const channels = new Array(nChannels);
    for (let i = 0; i < nChannels; ++i) {
        channels[i] = buffer.getChannelData(i);
    }
    return {
        channels,
        sampleRate: buffer.sampleRate,
        duration: buffer.duration
    };
}
class FeatureExtractor {
    constructor(audioBank) {
        this.audioBank = audioBank;
        this.client = new one_shot_1.OneShotExtractionClient(qmWorker, one_shot_1.OneShotExtractionScheme.REMOTE);
    }
    extract(request) {
        return this.client.collect(request);
    }
    getBeats(audioUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const beats = yield this.getFeature(audioUri, 'qm-vamp-plugins:qm-barbeattracker', 'beats');
            return beats.map(feature => ({
                time: { value: time_1.toSeconds(feature.timestamp) },
                label: { value: feature.label }
            }));
        });
    }
    getKeys(audioUri) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getFrameBasedFeature(audioUri, 'qm-vamp-plugins:qm-keydetector', 'tonic');
        });
    }
    getLoudnesses(audioUri) {
        return __awaiter(this, void 0, void 0, function* () {
            return undefined; /*this.getFrameBasedFeature(audioUri,
              'vamp-libextract:loudness', 'loudness');*/
        });
    }
    getFrameBasedFeature(audioUri, key, outputId) {
        return __awaiter(this, void 0, void 0, function* () {
            const feature = yield this.getFeature(audioUri, key, outputId);
            return feature.map(feature => ({
                time: { value: time_1.toSeconds(feature.timestamp) },
                value: feature.featureValues[0]
            }));
        });
    }
    getFeature(audioUri, key, outputId) {
        return __awaiter(this, void 0, void 0, function* () {
            const buffer = yield this.audioBank.getAudioBuffer(audioUri);
            return this.extractQmFeature(buffer, {
                key: key,
                outputId: outputId
            });
        });
    }
    extractQmFeature(buffer, feature) {
        const { channels, sampleRate } = bufferToAudioData(buffer);
        return this.extract({
            audioData: channels,
            audioFormat: {
                sampleRate,
                channelCount: channels.length
            },
            key: feature.key,
            outputId: feature.outputId
        }).then(response => response.features.collected);
    }
}
exports.FeatureExtractor = FeatureExtractor;
