import { DymoPlayer } from 'dymo-player';
import { DymoGenerator } from 'dymo-core';
import { Transition } from './types';
export declare const AVAILABLE_TRANSITIONS: string[];
export interface TransitionOptions {
    trackUri: string;
    cueOffset?: number;
    numBars?: number;
    duration?: number;
    position?: number;
}
export declare class MixGenerator {
    private generator;
    private player;
    private transitionObserverFunction;
    private mixDymoUri;
    private tracks;
    private store;
    private expressionGen;
    private transitionConstraints;
    constructor(generator: DymoGenerator, player: DymoPlayer, transitionObserverFunction: () => any);
    init(): Promise<void>;
    getMixDymo(): string;
    startMixWithFadeIn(options: TransitionOptions): Promise<Transition>;
    slam(options: TransitionOptions): Promise<Transition>;
    beatRepeat(options: TransitionOptions, times?: number): Promise<Transition>;
    echoFreeze(options: TransitionOptions): Promise<Transition>;
    effects(options: TransitionOptions): Promise<Transition>;
    powerDown(options: TransitionOptions): Promise<Transition>;
    crossfade(options: TransitionOptions): Promise<Transition>;
    beatmatchCrossfade(options: TransitionOptions): Promise<Transition>;
    private addPartsToMix;
    private addSilence;
    private addAligned;
    private addZipped;
    private applyBeatmatch;
    private applyFadeIn;
    private applyCrossfade;
    private addRandomBeatToLoop;
    transitionImmediatelyToRandomBars(trackUri: string, numBars?: number): Promise<any>;
    /**removes old track until current position + offset, registers new track and gets bars*/
    private initTransition;
    /**adds new track bars, loads constraints and controls, and returns transition object*/
    private endTransition;
    private registerTrackAndGetBars;
    private addRampWithTrigger;
    /** adds a trigger to init the the transition, calls an optional function when
      triggered */
    private addControlTrigger;
    private addConstraintTriggers;
    private makeCrossfade;
    private makeRampConstraint;
    private makeSetsConstraint;
    private getTempoFromBars;
    private getInitialBars;
    /**returns the last bars*/
    private findLastBar;
    private getTotalDuration;
    private getFeature;
    private getTransitionObject;
}
